@@ -1,7 +1,7 @@
 package BioX::Workflow;
 
 use 5.008_005;
-our $VERSION = '0.09';
+our $VERSION = '0.10';
 
 use Moose;
 use File::Find::Rule;
@@ -26,6 +26,9 @@ with 'MooseX::Getopt::Usage';
 with 'MooseX::Getopt::Usage::Role::Man';
 with 'MooseX::SimpleConfig';
 
+with 'MooseX::Object::Pluggable';
+
+
 # For pretty man pages!
 $ENV{TERM}='xterm-256color';
 
@@ -58,12 +61,12 @@ BioX::Workflow assumes your have a set of inputs, known as samples,
 and these inputs will carry on through your pipeline. There are some exceptions
 to this, which we will explore with the resample option.
 
-It also makes several assumtions about your output structure. It assumes you
+It also makes several assumptions about your output structure. It assumes you
 have each of your processes/rules outputting to a distinct directory.
 
 These directories will be created and automatically named based on your process
 name. You can disable this and make your own out directories by either
-specifiying auto_name: 1 in your global, in any of the local rules to disable
+specifiying auto_name: 0 in your global, in any of the local rules to disable
 it for that rule, or by specifying an outdirectory.
 
 =head1 A Simple Example
@@ -234,6 +237,29 @@ So on and so forth.
     # Workflow Finished
     #
 
+=head2 Finding your Samples
+
+Finding samples is the most crucial step of the workflow. If no samples are found, nothing is done.
+
+Normally, samples are files.
+
+    /path/to/indir
+        sample1.vcf
+        sample2.vcf
+        sample3.vcf
+
+But sometimes samples are directories.
+
+    /path/to/indir
+        /sample1
+            billions_of_small_files
+            from_the_Sequencer
+        /sample2
+            billions_of_small_files
+            from_the_Sequencer
+
+If this is the case with your workflow, please specify find_by_dir=>1.
+
 =head1 Customizing your output and special variables
 
 BioX::Workflow uses a few conventions and special variables. As you
@@ -251,7 +277,8 @@ define variables with other variables in this way. Everything is referenced
 with $self in order to dynamically pass variables to Text::Template. The sample
 variable, $sample, is the exception because it is defined in the loop. In
 addition you can create an OUTPUT/OUTPUT variables to clean up your process
-code.
+code. These are special variables that are also used in Drake. Please see L<BioX::Workflow::Plugin::Drake>
+for more details.
 
     ---
     global:
@@ -317,7 +344,9 @@ template. Make sure to use the previously defined $OUT. For more information see
 
 =head2 Directory Structure
 
-BioX::Workflow will create a directory structure based on your rule name, decamelized, and your globally defined outdir.
+BioX::Workflow will create a directory structure based on your rule name, and your globally defined outdir.
+
+=head3 Default Structure
 
 /path/to/outdir
     /rule1
@@ -327,6 +356,20 @@ BioX::Workflow will create a directory structure based on your rule name, decame
 If you don't like this you can globally disable auto_name (auto_name: 0), or simply defined indir or outdir within your global variables. If using the
 second method it is probably a good idea to also defined a ROOT_DIR in your global variables.
 
+=head3 By Sample Directory Structure
+
+Alternately you can create a directory structure that separates your rules into sample directories with by_sample_outdir=1
+
+/path/to/outdir
+    SAMPLE1/
+        /rule1
+        /rule2
+        /rule3
+    SAMPLE2/
+        /rule1
+        /rule2
+        /rule3
+
 =head2 Other variables
 
 A quick overview of other samples
@@ -366,6 +409,40 @@ check for uncompressed files, compress them, and then carry on with your life.
 The bgzip rule would first run a resample looking for only files ending in .vcf, and compress them. The following rule, normalize_snpeff, looks again
 in the indir (which we set here otherwise it would have been the previous rules outdir), and resamples based on the .vcf.gz extension.
 
+=head2 Plugins
+
+As of 0.10 there is a plugin system using L<MooseX::Object::Pluggable>
+
+    ---
+    plugins:
+        - FileDetails
+    global:
+        - indir: /home/user/gemini
+        - outdir: /home/user/gemini/gemini-wrapper
+        - file_rule: (.vcf)$|(.vcf.gz)$
+        - infile:
+    #So On and So Forth
+
+BioX::Workflow::Drake has been moved to BioX::Workflow::Plugin Drake. Instead of using
+
+biox-workflow-drake.pl --THINGS
+
+Instead add 'Drake' to your plugins list in your workflow file.
+
+=head3 Drake Plugin
+
+Drake is a 'make for data.' More information about it can be found here:
+L<https://github.com/Factual/drake> and the module can be found at L<BioX::Workflow::Plugin::Drake>.
+
+=head3 FileDetails Plugin
+
+BioX::Workflow will optionally put some commands at the end of your workflow to check files for
+metadata: MD5, DateTime created, last accessed, last modified, size, and human readable size.
+
+It creates a structure {$self->outdir}/meta/file.meta. The output structure will probably be changed in the future.
+
+For more information please see L<BioX::Workflow::Plugin::FileDetails>
+
 =head1 In Code Documenation
 
 You shouldn't really need to look here unless you have some reason to do some serious hacking.
@@ -388,6 +465,7 @@ then resampling based on the .vcf.gz extension.
 =cut
 
 has 'resample' => (
+    traits  => [ 'NoGetopt'  ],
     is => 'rw',
     isa => 'Bool',
     default => 0,
@@ -513,6 +591,7 @@ has 'wait' => (
      is => 'rw',
      isa => 'Bool',
      default => 1,
+     documentation => q(Print 'wait' at the end of each rule. If you are running as a plain bash script you probably don't need this.)
 );
 
 
@@ -527,6 +606,9 @@ has 'override_process' => (
      is => 'rw',
      isa => 'Bool',
      default => 0,
+     predicate => 'has_override_proccess',
+     clearer => 'clear_override_proccess',
+     documentation => q(Instead of for my $sample (@sample){ DO STUFF } just DOSTUFF),
 );
 
 =head3 indir outdir
@@ -539,6 +621,7 @@ has 'indir'  => (
     default => sub {getcwd();},
     predicate => 'has_indir',
     clearer => 'clear_indir',
+    documentation => q(Directory to look for samples),
 );
 
 has 'outdir'  => (
@@ -547,12 +630,28 @@ has 'outdir'  => (
     default => sub {getcwd();},
     predicate => 'has_outdir',
     clearer => 'clear_outdir',
+    documentation => q(Output directories for rules and processes),
 );
 
-=head3 Input Output
+=head3 create_outdir
+
+=cut
+
+has 'create_outdir' => (
+    is => 'rw',
+    isa => 'Bool',
+    predicate => 'has_create_outdir',
+    clearer => 'clear_create_outdir',
+    documentation => q(Create the outdir. You may want to turn this off if doing a rule that doesn't write anything, such as checking if files exist),
+    default => 1,
+);
+
+=head3 INPUT OUTPUT
 
 Special variables that can have input/output
 
+These variables are also used in L<BioX::Workflow::Plugin::Drake>
+
 =cut
 
 has 'OUTPUT' =>(
@@ -560,13 +659,16 @@ has 'OUTPUT' =>(
     isa => 'Str|Undef',
     predicate => 'has_OUTPUT',
     clearer => 'clear_OUTPUT',
+    documentation => q(Maybe clean up your code some. At the end of each process the OUTPUT becomes
+    the INPUT. Best when putting a single file through a stream of processes.)
 );
 
-has 'OUTPUT' =>(
+has 'INPUT' =>(
     is => 'rw',
     isa => 'Str|Undef',
-    predicate => 'has_OUTPUT',
-    clearer => 'clear_OUTPUT',
+    predicate => 'has_INPUT',
+    clearer => 'clear_INPUT',
+    documentation => q(See $OUTPUT)
 );
 
 =head3 file_rule
@@ -581,8 +683,12 @@ has 'file_rule' =>(
      default => sub { return "\\.[^.]*"; }
 );
 
+=head3 No GetOpt Here
+
+=cut
 
 has 'yaml' => (
+    traits  => [ 'NoGetopt'  ],
     is => 'rw',
 );
 
@@ -593,6 +699,7 @@ attributes read in from runtime
 =cut
 
 has 'attr' => (
+    traits  => [ 'NoGetopt'  ],
     is => 'rw',
     isa => 'Data::Pairs',
 );
@@ -604,6 +711,7 @@ Attributes defined in the global section of the yaml file
 =cut
 
 has 'global_attr' => (
+    traits  => [ 'NoGetopt'  ],
     is => 'rw',
     isa => 'Data::Pairs',
 );
@@ -615,6 +723,7 @@ Attributes defined in the rules->rulename->local section of the yaml file
 =cut
 
 has 'local_attr' => (
+    traits  => [ 'NoGetopt'  ],
     is => 'rw',
     isa => 'Data::Pairs',
 );
@@ -624,6 +733,7 @@ has 'local_attr' => (
 =cut
 
 has 'local_rule' => (
+    traits  => [ 'NoGetopt'  ],
     is => 'rw',
     isa => 'HashRef'
 );
@@ -635,6 +745,7 @@ Infiles to be processed
 =cut
 
 has 'infiles' => (
+    traits  => [ 'NoGetopt'  ],
      is => 'rw',
      isa => 'ArrayRef',
 );
@@ -655,6 +766,7 @@ Do stuff
 =cut
 
 has 'process' => (
+    traits  => [ 'NoGetopt'  ],
     is => 'rw',
     isa => 'Str',
 );
@@ -666,6 +778,7 @@ Do stuff
 =cut
 
 has 'key' => (
+    traits  => [ 'NoGetopt'  ],
     is => 'rw',
     isa => 'Str',
 );
@@ -731,6 +844,7 @@ sub run {
     $self->yaml($array);
 
     $self->class_load;
+    $self->plugin_load;
 
     $self->global_attr(Data::Pairs->new($array->{global}));
 
@@ -767,6 +881,8 @@ Set initial indir and outdir
 sub make_outdir {
     my($self) = @_;
 
+    return unless $self->create_outdir;
+
     make_path($self->outdir) if ! -d $self->outdir;
 }
 
@@ -796,6 +912,8 @@ sub get_samples{
     my($self) = shift;
     my(@whole, @basename, $text);
 
+    return if $self->attr->exists('samples');
+
     $text = $self->file_rule;
 
     if($self->find_by_dir){
@@ -818,7 +936,26 @@ sub get_samples{
     }
 }
 
-=head3 load
+
+=head3 plugin_load
+
+Load plugins defined in yaml with MooseX::Object::Pluggable
+
+=cut
+
+sub plugin_load {
+    my($self) = shift;
+
+    return unless $self->yaml->{plugins};
+
+    my $modules = $self->yaml->{plugins};
+
+    foreach my $m (@$modules){
+       $self->load_plugin($m);
+    }
+}
+
+=head3 class_load
 
 Load classes defined in yaml with Class::Load
 
@@ -890,7 +1027,7 @@ sub eval_attr {
     my $self = shift;
     my $sample = shift;
 
-    $DB::single=2;
+    #$DB::single=2;
     my @keys = $self->attr->get_keys();
 
     foreach my $k (@keys){
@@ -907,11 +1044,12 @@ sub eval_attr {
             $text = $template->fill_in(HASH => {self => \$self});
         }
 
-        $DB::single=2;
+        #$DB::single=2;
         $self->$k($text);
     }
 
-    $self->make_outdir if $self->attr->exists('OUTPUT')
+    #$self->make_outdir if $self->attr->exists('OUTPUT');
+    $self->make_outdir if $self->create_outdir;
 }
 
 sub clear_attr {
@@ -981,7 +1119,6 @@ sub dothings {
 
     $key = $keys[0];
     $self->key($key);
-    #$camel_key = decamelize($key);
     $camel_key= $key;
 
     if($self->auto_name){
@@ -1022,8 +1159,9 @@ sub dothings {
 
             print "$self->{comment_char} Local Variables:\n";
 
-            if($self->auto_input && $self->has_OUTPUT){
-                $self->local_attr->set('OUTPUT' => $self->OUTPUT);
+            if($self->auto_input ){
+                $self->local_attr->set('OUTPUT' => $self->OUTPUT) if $self->has_OUTPUT;
+                $self->local_attr->set('INPUT' => $self->global_attr->get_values('INPUT')) if $self->global_attr->exists('INPUT');
             }
 
             my @keys = $self->local_attr->get_keys();
@@ -1108,39 +1246,11 @@ sub write_process{
 
     if(!$self->override_process){
         foreach my $sample (@{$self->samples}){
-            if($self->by_sample_outdir){
-                my($tt, $key);
-                $tt = $self->outdir;
-                $key = $self->key;
-                $tt =~ s/$key/$sample\/$key/;
-                $self->outdir($tt);
-                $self->make_outdir;
-                $DB::single=2;
-
-                $tt = $self->indir;
-                if($tt =~ m/\{\$self/){
-                    $DB::single=2;
-                    $tt = "$tt/{\$sample}";
-                    $self->indir($tt);
-                    $self->attr->set('indir' => $self->indir) if $self->attr->exists('indir');
-                }
-                elsif($self->has_pkey){
-                    $DB::single=2;
-                    $key = $self->pkey;
-                    $tt =~ s/$key/$sample\/$key/;
-                    $self->indir($tt);
-                    $self->attr->set('indir' => $self->indir) if $self->attr->exists('indir');
-                }
-                else{
-                    $DB::single=2;
-                    $tt = "$tt/$sample";
-                    $self->indir($tt);
-                    $self->attr->set('indir' => $self->indir) if $self->attr->exists('indir');
-                }
-            }
+            $self->process_by_sample_outdir($sample) if $self->by_sample_outdir;
+            $DB::single=2;
             $self->eval_attr($sample);
             my $data = {self => \$self, sample => $sample};
-            $DB::single=2;
+            #$DB::single=2;
             $self->process_template($data);
             $self->outdir($origout);
             $self->indir($origin);
@@ -1156,21 +1266,74 @@ sub write_process{
         print "\nwait\n";
     }
 
+    $self->OUTPUT_to_INPUT;
 
+    $self->pkey($self->key);
+}
+
+=head3 process_by_sample_outdir
+
+Make sure indir/outdirs are named appropriated for samples when using by
+
+=cut
+
+sub process_by_sample_outdir {
+    my $self = shift;
+    my $sample = shift;
+
+    my($tt, $key);
+    $tt = $self->outdir;
+    $key = $self->key;
+    $tt =~ s/$key/$sample\/$key/;
+    $self->outdir($tt);
+    $self->make_outdir;
     $DB::single=2;
+
+    $tt = $self->indir;
+    if($tt =~ m/\{\$self/){
+        $DB::single=2;
+        $tt = "$tt/{\$sample}";
+        $self->indir($tt);
+        $self->attr->set('indir' => $self->indir) if $self->attr->exists('indir');
+    }
+    elsif($self->has_pkey){
+        $DB::single=2;
+        $key = $self->pkey;
+        $tt =~ s/$key/$sample\/$key/;
+        $self->indir($tt);
+        $self->attr->set('indir' => $self->indir) if $self->attr->exists('indir');
+    }
+    else{
+        $DB::single=2;
+        $tt = "$tt/$sample";
+        $self->indir($tt);
+        $self->attr->set('indir' => $self->indir) if $self->attr->exists('indir');
+    }
+}
+
+
+=head3 OUTPUT_to_INPUT
+
+If we are using auto_input chain INPUT/OUTPUT
+
+=cut
+
+sub OUTPUT_to_INPUT {
+    my $self = shift;
+    $DB::single=2;
+    #Change the output to input
     if($self->auto_input && $self->local_attr->exists('OUTPUT')){
         $DB::single=2;
         my($tmp, $indir, $outdir) = ($self->local_attr->get_values('OUTPUT'), $self->indir, $self->outdir);
-        $tmp =~ s/$outdir/$indir/;
-        $self->OUTPUT($tmp);
+        $tmp =~ s/{\$self->outdir}/{\$self->indir}/g;
+        $self->INPUT($tmp);
+        #This is not the best way of doing this....
+        $self->global_attr->set(INPUT => $self->INPUT);
         $DB::single=2;
     }
     else{
-        $self->OUTPUT('');
+        $self->clear_OUTPUT();
     }
-    $self->OUTPUT('');
-
-    $self->pkey($self->key);
 }
 
 sub process_template{
@@ -1182,8 +1345,8 @@ sub process_template{
     $DB::single=2;
     $template->fill_in(HASH => $data, OUTPUT => \*STDOUT);
 
-    #$self->OUTPUT($self->local_attr->get_values('OUTPUT')) if $self->local_attr->exists('OUTPUT');
-    #$self->OUTPUT($self->local_attr->get_values('OUTPUT')) if $self->local_attr->exists('OUTPUT');
+    $self->INPUT($self->local_attr->get_values('INPUT')) if $self->local_attr->exists('INPUT');
+    $self->OUTPUT($self->local_attr->get_values('OUTPUT')) if $self->local_attr->exists('OUTPUT');
 
     print "\n\n";
 }
